### script to define and add consensus genotype according to the available replicates

#### collect and check arguments provided
args <- commandArgs(TRUE)

if (length(args)<5 | length(args)>5)
{
	print ("##### Error Message #####")
	print ("you need to provide five arguments in this order :")
	print ("1- The number of repeats for each sample")
	print ("2- The number of samples")
	print ("3- The threshold to apply to seq count")
	print ("4- The homozygous threshold")
	print ("5- The name of the input tab file")	
	stop()
}

repet=as.numeric(args[1])
NbEch=as.numeric(args[2])
threshold=as.numeric(args[3])
homozygousThreshold=as.numeric(args[4])
filename=args[5]



### read  data file 
data=read.table(filename,h=T,stringsAsFactors=FALSE)

for (i in 1:NbEch)
{
	### get allele list for each replicate, filtered according to the fixed threshold
	AlleleList=list(c())
	for (j in 1:repet) 
	{
		#print (j)
		if (is.na(data[((i-1)*(repet+1))+j,1])) 
		{
			AlleleList[j]=list(c(NA))
		}
		else
		{
			liste=list(c())
			#liste=list()
			listCount=list(c())
			#listCount=list()
			liste[1]=data[((i-1)*(repet+1))+j,1]
			listCount[1]=data[((i-1)*(repet+1))+j,7]
			cpt=8
			go=1
			while (cpt<13 & go==1)
			{
				if (!is.na(data[((i-1)*(repet+1))+j,cpt]))
				{
					if (data[((i-1)*(repet+1))+j,cpt]>=(threshold*data[((i-1)*(repet+1))+j,7]))
					{
						# get the size of the current allele
						splitname=strsplit(data[((i-1)*(repet+1))+j,cpt-6],'_')
						currentAlleleLength=as.numeric(splitname[[1]][1])						
						
					
						go2=1
						# check for allele of the same size
						listLength=length(liste)
						for (l in 1:listLength)
						{
							splitname=strsplit(liste[[l]][1],'_')
							AlleleLength=as.numeric(splitname[[1]][1])
							
							if (currentAlleleLength==AlleleLength)
							{
								go2=0
								if (data[((i-1)*(repet+1))+j,cpt]>0.3*listCount[[l]][1])
								{
									liste[listLength+1]=data[((i-1)*(repet+1))+j,cpt-6]
									listCount[listLength+1]=data[((i-1)*(repet+1))+j,cpt]
								}
							}
							else
							{
								#liste[listLength+1]=data[((i-1)*(repet+1))+j,cpt-6]
								#listCount[listLength+1]=data[((i-1)*(repet+1))+j,cpt]
							}
							
							
								
						}
					
						if (go2==1)
						{
							liste[listLength+1]=data[((i-1)*(repet+1))+j,cpt-6]
							listCount[listLength+1]=data[((i-1)*(repet+1))+j,cpt]
						}
					
						cpt=cpt+1
						
					}
					else
					{
						go=0
					}
				}
				else
				{
					go=0
				}
			}
			
			AlleleList[[j]]=liste
		}
		
	}	
	
	### concat allele list
	Alleles=unlist(AlleleList)
	tabAlleles=sort(table(Alleles),decreasing=TRUE)
	
	if (length(tabAlleles)==0)
	{
		data[((i-1)*(repet+1))+(repet+1),1]=0
		data[((i-1)*(repet+1))+(repet+1),2]=0
	} 
	else
	{
		### select only alleles with count>=2
		tabAllelesSelect=tabAlleles[tabAlleles>=2]
		
		if (length(tabAllelesSelect)==0)
		{
			data[((i-1)*(repet+1))+(repet+1),1]=0
			data[((i-1)*(repet+1))+(repet+1),2]=0
		}

		if (length(tabAllelesSelect)==1) 
		{
			if (tabAllelesSelect[1]>=homozygousThreshold*repet) 
			{
				data[((i-1)*(repet+1))+(repet+1),1]=names(tabAllelesSelect)
				data[((i-1)*(repet+1))+(repet+1),2]=names(tabAllelesSelect)
				# quality
				data[((i-1)*(repet+1))+(repet+1),3]=tabAllelesSelect[1]/repet
				
				
			}
			else
			{
				data[((i-1)*(repet+1))+(repet+1),1]=0
				data[((i-1)*(repet+1))+(repet+1),2]=0
			}
		}
		
		if (length(tabAllelesSelect)==2)
		{
			all1=names(tabAllelesSelect)[1]
			all2=names(tabAllelesSelect)[2]
			len_all1=as.numeric(strsplit(all1,'_')[[1]][1])
			len_all2=as.numeric(strsplit(all2,'_')[[1]][1])
			if (len_all2<len_all1)
			{
				data[((i-1)*(repet+1))+(repet+1),1]=all2
				data[((i-1)*(repet+1))+(repet+1),2]=all1
			}
			else 
			{
				data[((i-1)*(repet+1))+(repet+1),1]=all1
				data[((i-1)*(repet+1))+(repet+1),2]=all2
			}
			
		
			#data[((i-1)*(repet+1))+(repet+1),1]=names(tabAllelesSelect)[1]
			#data[((i-1)*(repet+1))+(repet+1),2]=names(tabAllelesSelect)[2]
			
			# quality
			allelesNames=names(tabAllelesSelect)
			genoCount=0
			for (n in 1:repet)
			{
				if (is.element(allelesNames[1],AlleleList[[n]]) & is.element(allelesNames[2],AlleleList[[n]]))
				{
					genoCount=genoCount+1
				}		
			}
			data[((i-1)*(repet+1))+(repet+1),3]=genoCount/repet
			
			
			
			
		}
		
		if (length(tabAllelesSelect)>2)
		{
			data[((i-1)*(repet+1))+(repet+1),1]=1
			data[((i-1)*(repet+1))+(repet+1),2]=1
		}
		
	}
}


### print updated table
fileoutname=paste("c",filename,sep="")
write.table(data,fileoutname,quote=FALSE)


### create table_cns
table_cns=data[seq(repet+1,NbEch*(repet+1),(repet+1)),]
table_cns=table_cns[,1:3]
colnames(table_cns)=c("allele1","allele2","quality")
fileoutname2=paste("cns_",filename,sep="")
write.table(table_cns,fileoutname2,quote=FALSE)
 

### check for ref file
parts=strsplit(filename,'_')
reffilename=paste(substr(parts[[1]][3],1,2),'_',parts[[1]][4],'_ref.txt',sep="")
if (file.exists(reffilename))
{
	#### modify cns table allele names according to ref file
	# read ref file
	#parts=strsplit(filename,'_')
	#reffilename=paste(substr(parts[[1]][3],1,2),'_',parts[[1]][4],'_ref.txt',sep="")
	refAlleles=read.table(reffilename,h=T,stringsAsFactors=FALSE)
	
	# create filename to store allele name changes
	allele_changes_filename=paste(parts[[1]][3],'_',parts[[1]][4],"_allele_changes.txt",sep="")
	cat("sample\toldname\tnewname",file=allele_changes_filename,sep="\n")

	# read data serie file
	parts=strsplit(filename,'_')
	seriefilename=paste(parts[[1]][2],'_',parts[[1]][3],'_',parts[[1]][4],'_',parts[[1]][5],sep="")
	dataserie=read.table(seriefilename,h=T)
	

	# parse cns table and check each allele
	table_cnsh=table_cns
	for (i in 1:NbEch) 
	#for (i in 1:12)
	{
		for (j in 1:2)
		{
			#print (i)
			#print (j)
			allele=table_cnsh[i,j]
			#print (allele)
			if (allele!='0' & allele!='1')
			{
				size=as.numeric(strsplit(allele,'_')[[1]][1])
				serie=as.numeric(strsplit(allele,'_')[[1]][2])
				sub=dataserie[dataserie$seq_length==size & dataserie$series==serie,]
				seq=as.character(sub$sequence)
				found=0
				namefound=0
				cpt=1
				while (cpt<=(dim(refAlleles)[1]) & found==0)
				{
					if (seq==refAlleles[cpt,2])
					{
						found=1
						if (allele!=refAlleles[cpt,1])
						{
							table_cnsh[i,j]=refAlleles[cpt,1]
							cat(rownames(table_cnsh)[i],allele,refAlleles[cpt,1],"\n",file=allele_changes_filename,sep="\t",append=TRUE)
						}
					}
					cpt=cpt+1
				}
				if (found==0)
				{
					refAllelesNames=refAlleles[,1]
					if (!is.element(allele,refAllelesNames))
					{
						refAlleles=rbind(refAlleles,c(allele,seq))
					}
					else
					{
						ct=1
						new_name=paste(size,'_',strtoi(ct),sep='')
						while (is.element(new_name,refAllelesNames))
						{
							ct=ct+1
							new_name=paste(size,'_',strtoi(ct),sep='')
						}
						refAlleles=rbind(refAlleles,c(new_name,seq))
						table_cnsh[i,j]=new_name			
					}
		
				}
			}
		}
	
	}
	### write table_cnsh
	fileoutnamecnsh=paste("cnsh_",filename,sep="")
	write.table(table_cnsh,fileoutnamecnsh,quote=FALSE)

	### write updated table refAlleles
	write.table(refAlleles,reffilename,quote=FALSE,row.names=FALSE)
 
}
if (!file.exists(reffilename))
{
	### create allele ref file
	refAlleles=c(table_cns$allele1,table_cns$allele2)
	refAlleles=unique(refAlleles)
	refAlleles=refAlleles[refAlleles!=1 & refAlleles!=0]

	### read serie file
	parts=strsplit(filename,'_')
	seriefilename=paste(parts[[1]][2],'_',parts[[1]][3],'_',parts[[1]][4],'_',parts[[1]][5],sep="")
	dataserie=read.table(seriefilename,h=T)
	#dataserie=read.table(fileseriename,h=T)

	seqs=c()
	for (i in 1:(length(refAlleles)))
	{
		size=as.numeric(strsplit(refAlleles[i],'_')[[1]][1])
		serie=as.numeric(strsplit(refAlleles[i],'_')[[1]][2])
		sub=dataserie[dataserie$seq_length==size & dataserie$series==serie,]
		seqs=c(seqs,as.character(sub$sequence))
	}
	dfRefAlleles=data.frame(refAlleles,seqs)

	### write dfRefAlleles to ref file
	tmp=strsplit(filename,'_')
	reffilename=paste(substr(tmp[[1]][3],1,2),'_',strtoi(tmp[[1]][4]),"_ref.txt",sep="")
	write.table(dfRefAlleles,reffilename,quote=FALSE,row.names=FALSE)
	
	### write table_cnsh
	fileoutnamecnsh=paste("cnsh_",filename,sep="")
	write.table(table_cns,fileoutnamecnsh,quote=FALSE)
}



